package cn.rankin.cmsweb.controller.user;

import cn.rankin.cmsweb.code.CmsWebAPICode;
import cn.rankin.cmsweb.service.product.RecommendService;
import cn.rankin.cmsweb.service.user.MerchantService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.user.MerchantDTO;
import cn.rankin.common.utils.dto.user.MerchantSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.common.utils.vo.product.RecommendVo;
import cn.rankin.common.utils.vo.user.MerchantVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(value = "/merchant")
public class MerchantController {

    @Autowired
    private MerchantService merchantService;

    @Autowired
    private RecommendService recommendService;

    @RequestMapping(value = {"", "/list"}, method = RequestMethod.GET)
    public APIResult<Page<MerchantVo>> getPage(MerchantSearchDTO searchDTO) {
        return merchantService.search(BeanUtil.convertToMap(searchDTO));
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<MerchantVo> create(@RequestBody @Valid MerchantDTO merchantDTO) {
        return merchantService.create(merchantDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<MerchantVo> update(@RequestBody @Valid MerchantDTO merchantDTO) {
        APIResult<MerchantVo> result = merchantService.update(merchantDTO);
        if (!result.getSuccess()) {
            return result;
        }

        MerchantVo merchantVo = result.getData();
        List<String> recommendIdList = merchantDTO.getRecommendIdList();

        if (!CollectionUtils.isEmpty(recommendIdList)) {
            APIResult<List<RecommendVo>> recommendAPIResult = recommendService.put(merchantDTO.getId(), recommendIdList);
            if (!recommendAPIResult.getSuccess()) {
                return APIResult.error(CmsWebAPICode.error("更新推荐位失败"));
            }
            merchantVo.setRecommendVoList(recommendAPIResult.getData());
        }

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable String id) {
        return merchantService.delete(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<MerchantVo> getMerchant(@PathVariable String id) {
        APIResult<MerchantVo> result = merchantService.getMerchant(id);
        if (result.getSuccess()) {
            MerchantVo merchantVo = result.getData();
            APIResult<List<RecommendVo>> recommendAPIResult = recommendService.get(id);
            if (recommendAPIResult.getSuccess()) {
                merchantVo.setRecommendVoList(recommendAPIResult.getData());
            }
        }
        return result;
    }

}
