package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.GroupDTO;
import cn.rankin.data.api.product.dto.TagDTO;
import cn.rankin.common.utils.dto.search.GroupSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.TagGroup;
import cn.rankin.data.api.product.entity.Tag;
import cn.rankin.productservice.service.TagGroupService;
import cn.rankin.productservice.service.TagService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "group")
public class GroupController {

    @Autowired
    private TagGroupService tagGroupService;

    @Autowired
    private TagService tagService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<TagGroup>> search(GroupSearchDTO searchDTO) {
        TagGroup tagGroup = new TagGroup();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            tagGroup.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            tagGroup.setName("%" + name + "%");
        }

        // others
        tagGroup.setStatus(searchDTO.getStatus());
        tagGroup.setMerchantId(searchDTO.getMerchantId());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return tagGroupService.search(tagGroup, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<TagGroup> create(@Valid @RequestBody GroupDTO groupDTO) {

        APIResult<TagGroup> result = tagGroupService.create(groupDTO);
        if (!result.getSuccess()) {
            return result;
        }

        TagGroup tagGroup = result.getData();

        List<TagDTO> tagDTOList = groupDTO.getTagList();
        if (!CollectionUtils.isEmpty(tagDTOList)) {
            APIResult<List<Tag>> tagResult = tagService.updateGroupRelation(tagGroup.getId(), tagDTOList);
            tagGroup.setTagList(tagResult.getData());
        }

        return result;
    }

    @Transactional
    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<TagGroup> update(@Valid @RequestBody GroupDTO groupDTO) {
        APIResult<TagGroup> result = tagGroupService.update(groupDTO);
        if (!result.getSuccess()) {
            return result;
        }

        TagGroup tagGroup = result.getData();

        List<TagDTO> tagDTOList = groupDTO.getTagList();
        if (tagDTOList != null) {
            APIResult<List<Tag>> tagResult = tagService.updateGroupRelation(tagGroup.getId(), tagDTOList);
            if (!tagResult.getSuccess()) {
                return APIResult.error(new BaseCode(tagResult.getCode(), tagResult.getMessage()));
            }
            tagGroup.setTagList(tagResult.getData());
        }

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String id) {
        APIResult<Boolean> groupResult = tagGroupService.delete(id);
        if (!groupResult.getSuccess()) {
            return groupResult;
        }

        // 同时删除下属所有tag
        return tagService.deleteByGroupId(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<TagGroup> getGroup(@PathVariable("id") String id) {
        APIResult<TagGroup> result = tagGroupService.findNormalById(id);
        TagGroup tagGroup = result.getData();
        if (!result.getSuccess() || tagGroup == null) {
            return result;
        }

        APIResult<List<Tag>> tagResult = tagService.findNormalByGroupId(id);
        if (!tagResult.getSuccess()) {
            return APIResult.error(new BaseCode(tagResult.getCode(), tagResult.getMessage()));
        }

        tagGroup.setTagList(tagResult.getData());
        return result;

    }

}
