package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.MerchantProductSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.productservice.proxy.MerchantProductServiceProxy;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedHashMap;

@RestController
@RequestMapping(value = "/merchant/products")
public class MerchantProductController {

    @Autowired
    private MerchantProductServiceProxy merchantProductServiceProxy;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<Page<MerchantProduct>> search(MerchantProductSearchDTO searchDTO) {
        MerchantProduct merchantProduct = new MerchantProduct();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            merchantProduct.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            merchantProduct.setName("%" + name + "%");
        }

        // others
        merchantProduct.setType(searchDTO.getType());
        merchantProduct.setMerchantId(searchDTO.getMerchantId());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return merchantProductServiceProxy.search(merchantProduct, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<MerchantProduct> getProduct(@PathVariable("id") String productId) {
        return merchantProductServiceProxy.getProduct(productId);
    }

}
