package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.search.SearchDTO;
import cn.rankin.data.api.product.dto.SupportDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.Support;
import cn.rankin.productservice.proxy.SupportServiceProxy;
import cn.rankin.productservice.service.SupportReferenceService;
import cn.rankin.productservice.service.SupportService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "support")
public class SupportController {

    @Autowired
    private SupportService supportService;

    @Autowired
    private SupportReferenceService selfRelationService;

    @Autowired
    private SupportServiceProxy supportServiceProxy;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<Support>> search(SearchDTO searchDTO) {
        Support support = new Support();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            support.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            support.setName("%" + name + "%");
        }

        support.setStatus(searchDTO.getStatus());
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return supportService.search(support, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<Support> getSupport(@PathVariable("id") String id) {
        if (StringUtils.isEmpty(id)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        APIResult<Support> result = supportService.findById(id);
        if (!result.getSuccess()) {
            return result;
        }

        Support support = result.getData();

        APIResult<List<Support>> supportResult = supportServiceProxy.getRelationSupport(support.getId());
        support.setSupportList(supportResult.getData());

        return result;

    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Support> create(@RequestBody SupportDTO supportDTO) {
        String code = supportDTO.getCode();
        if (StringUtils.isEmpty(code)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        APIResult<Support> result = supportService.create(supportDTO);
        Support support = result.getData();
        List<String> supportIdList = supportDTO.getSupportList();
        if (supportIdList == null || supportIdList.size() == 0 || support == null) {
            return result;
        }

        APIResult<List<Support>> supportResult = supportServiceProxy.updateSelfRelation(support.getId(), supportIdList);
        if (!supportResult.getSuccess()) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return APIResult.error(new BaseCode(supportResult.getCode(), supportResult.getMessage()));
        }
        support.setSupportList(supportResult.getData());

        return result;

    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Support> update(@RequestBody SupportDTO supportDTO) {
        if (supportDTO.getId() == null) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        APIResult<Support> result = supportService.update(supportDTO);
        Support support = result.getData();

        List<String> supportIdList = supportDTO.getSupportList();
        if (supportIdList == null || support == null) {
            return result;
        }else if (supportIdList.size() == 0) {
            selfRelationService.deleteBySupportId(support.getId());
            return result;
        }

        APIResult<List<Support>> supportResult = supportServiceProxy.updateSelfRelation(support.getId(), supportIdList);
        if (!supportResult.getSuccess()) {
            return APIResult.error(new BaseCode(supportResult.getCode(), supportResult.getMessage()));
        }
        support.setSupportList(supportResult.getData());

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String id) {
        if (StringUtils.isEmpty(id)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        return supportService.delete(id);
    }
}
