package cn.rankin.productservice.proxy;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.data.api.product.entity.Goods;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.productservice.repository.GoodsRepository;
import cn.rankin.productservice.repository.MerchantProductRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Service
public class MerchantProductServiceProxy {

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    @Autowired
    private GoodsRepository goodsRepository;

    public APIResult<Page<MerchantProduct>> search(MerchantProduct merchantProduct, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = merchantProductRepository.count(merchantProduct);
        Page<MerchantProduct> page = new Page<>(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<MerchantProduct> merchantProductList = merchantProductRepository.find(merchantProduct, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        if (merchantProductList.size() == 0) {
            page.setList(new ArrayList<>());
            return APIResult.ok(page);
        }

        List<String> productIdList = new ArrayList<>();
        merchantProductList.forEach(e -> productIdList.add(e.getPid()));
        List<Goods> goodsList = goodsRepository.findByPidIn(productIdList);

        Map<String, List<Goods>> goodsMap = new HashMap<>();
        goodsList.forEach(good -> {
            String productId = good.getPid();
            if (goodsMap.containsKey(productId)) {
                goodsMap.get(productId).add(good);
            }else {
                List<Goods> tmpList = new ArrayList<>();
                tmpList.add(good);
                goodsMap.put(productId, tmpList);
            }
        });

        // 按sort排序
        for (String productId: goodsMap.keySet()) {
            List<Goods> goods = goodsMap.get(productId);
            Collections.sort(goods, new Comparator<Goods>() {
                @Override
                public int compare(Goods o1, Goods o2) {
                    return o1.getSort().compareTo(o2.getSort());
                }
            });
        }

        for (MerchantProduct product : merchantProductList) {
            String productId = product.getPid();
            List<Goods> goods = goodsMap.get(productId);
            if (!CollectionUtils.isEmpty(goods)) {
                product.setGoods(goods);
            }
        }

        page.setList(merchantProductList);

        return APIResult.ok(page);
    }

    public APIResult<MerchantProduct> getProduct(String productId) {
        MerchantProduct merchantProduct = merchantProductRepository.findByPid(productId);

        if (merchantProduct == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        List<Goods> goodsList = goodsRepository.findByPid(productId);
        merchantProduct.setGoods(goodsList);

        return APIResult.ok(merchantProduct);
    }
}
