package cn.rankin.productservice.proxy;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.product.entity.Support;
import cn.rankin.data.api.product.entity.SupportReference;
import cn.rankin.productservice.service.SupportReferenceService;
import cn.rankin.productservice.service.SupportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Service
public class SupportServiceProxy {

    @Autowired
    private SupportService supportService;

    @Autowired
    private SupportReferenceService selfRelationService;

    public Support getSupport(String id) {
        APIResult<Support> result = supportService.findById(id);
        Support support = result.getData();
        if (!result.getSuccess() || support == null) {
            return support;
        }

        APIResult<List<Support>> supportResult = getRelationSupport(support.getId());
        support.setSupportList(supportResult.getData());

        return support;
    }

    @Transactional
    public APIResult<List<Support>> updateSelfRelation(String fid, List<String> tids) {
        if (tids == null) {
            return APIResult.ok();
        }else if (tids.size() == 0) {
            return APIResult.ok(new ArrayList<Support>());
        }

        APIResult<List<Support>> supportResult = supportService.findByIds(tids);
        List<Support> supportList = supportResult.getData();
        if (supportList == null || supportList.size() != tids.size()) {
            return APIResult.error(APICode.error("存在错误的周边id"));
        }

        selfRelationService.update(fid, tids);

        return APIResult.ok(supportList);
    }

    public APIResult<List<Support>> getRelationSupport(String fid) {
        List<SupportReference> relationList = selfRelationService.findByFid(fid);
        if (relationList == null || relationList.size() == 0) {
            return APIResult.ok(new ArrayList<>());
        }

        List<String> supportIdList = new ArrayList<>();
        relationList.forEach( relation -> supportIdList.add(relation.getTid()));

        APIResult<List<Support>> supportResult = supportService.findByIds(supportIdList);

        return APIResult.ok(supportResult.getData());
    }
}
