package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.product.entity.ProductTagRelation;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

public interface ProductTagRelationRepository extends BasicJpaRepository<ProductTagRelation, String> {

    List<ProductTagRelation> findAllByPid(String itemId);

    @Query(value = "select r from ProductTagRelation r where r.tagId = :tagId and r.status = :status")
    List<ProductTagRelation> findByTagId(@Param("tagId") String tagId, @Param("status") BaseStatusEnum status);

    List<ProductTagRelation> findAllByTagId(String tagId);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update ProductTagRelation r set r.status = :status where r.pid = :itemId")
    Integer deleteByItemId(@Param("itemId") String itemId, @Param("status")BaseStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update ProductTagRelation r set r.status = :status where r.tagId = :tagId")
    Integer deleteByTagId(@Param("tagId") String tagId, @Param("status") BaseStatusEnum status);
}
