package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.product.entity.Tag;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


public interface TagRepository extends BasicJpaRepository<Tag, String> {

    String SQL = "select new Tag(t.id, t.name, t.type, t.groupId, t.sort, t.status, t.gmtCreated, t.gmtModified, g.name, g.merchantId) from Tag t, TagGroup g where t.groupId = g.id";

    List<Tag> search(Tag tag, Integer start, Integer size);

    @Query(value = SQL + " and t.groupId = :groupId and t.status = :status")
    List<Tag> findNormalByGroupId(@Param("groupId") String groupId, @Param("status") BaseStatusEnum status);

    @Query(value = SQL + " and t.id in (:ids) and t.status = :status order by t.sort asc")
    List<Tag> findByIdIn(@Param("ids") List<String> ids, @Param("status") BaseStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update Tag t set t.status = :status where t.id = :id")
    Integer updateStatusById(@Param("id") String id, @Param("status") BaseStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update Tag t set t.status = :status where t.groupId = :groupId")
    Integer deleteByGroupId(@Param("groupId") String groupId, @Param("status") BaseStatusEnum status);
}
