package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.CourseDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.product.entity.Course;
import cn.rankin.productservice.repository.*;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.LinkedHashMap;
import java.util.List;

@Slf4j
@Service
public class CourseService {

    @Autowired
    private CourseRepository courseRepository;

    public boolean exists(String code) {
        Long count = courseRepository.countByCode(code);
        return count > 0L;
    }

    public APIResult<Page<Course>> search(Course course, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = courseRepository.count(course);
        Page<Course> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Course> wareList = courseRepository.find(course, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(wareList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<Course> create(CourseDTO courseDTO) {
        String code = courseDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Course course = DTOConverter.convert(courseDTO);
        Course result = courseRepository.save(course);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Course> update(CourseDTO courseDTO) {

        Course course = DTOConverter.convert(courseDTO);
        Course result = courseRepository.update(course);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = courseRepository.deleteById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<Course> findById(String id) {
        Course course = courseRepository.findById(id);
        if (course == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }

        return APIResult.ok(course);
    }

    public APIResult<List<Course>> findAllByIds(List<String> courseIdList) {
//        List<Course> courseList = courseRepository.findByIdIn(courseIdList, BaseStatusEnum.NORMAL);
        List<Course> courseList = courseRepository.findByIds(courseIdList);
        return APIResult.ok(courseIdList);
    }
}
