package cn.rankin.productservice.service;

import cn.rankin.data.api.product.dto.CourseSubRelationDTO;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.CourseSubRelation;
import cn.rankin.productservice.repository.CourseSubRelationRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class CourseSubRelationService {

    @Autowired
    private CourseSubRelationRepository subRelationRepository;

    public List<CourseSubRelation> findByLessonId(String lessonId) {
        return subRelationRepository.findBySubId(lessonId, BaseStatusEnum.NORMAL);
    }

    public List<CourseSubRelation> findByCourseId(String coursId) {
        return subRelationRepository.findByCourseId(coursId, BaseStatusEnum.NORMAL);
    }

    @Transactional
    public Integer deleteByCourseId(String courseId) {
        return subRelationRepository.deleteByCourseId(courseId, BaseStatusEnum.DEL);
    }

    @Transactional
    public void update(String courseId, List<CourseSubRelationDTO> subRelationDTOList) {
        if (subRelationDTOList == null) {
            return;
        }else if (subRelationDTOList.size() == 0) {
            deleteByCourseId(courseId);
            return;
        }

        // 记录顺序数组: 最新的
        List<String> subIdList = new ArrayList<>();

        subRelationDTOList.forEach( e -> {
            String subId = e.getId();
            subIdList.add(subId);
        });

        // 更新现有的, 同时记录现有的关系
        List<String> currentIdList = new ArrayList<>();
        List<CourseSubRelation> currentRelationList = subRelationRepository.findAllByCourseId(courseId);
        currentRelationList.forEach( e -> {
            String subId = e.getSubId();
            if (!subIdList.contains(subId)) {
                e.setStatus(BaseStatusEnum.DEL);
            }else {
                e.setStatus(BaseStatusEnum.NORMAL);
            }
            currentIdList.add(subId);
        });

        // 新加的
        List<String> newAddSubIdList = ListUtil.subtract(subIdList, currentIdList);
        if (newAddSubIdList != null && newAddSubIdList.size() > 0) {
            subRelationDTOList.forEach( e -> {
                String subId = e.getId();
                currentRelationList.add(convert(courseId, e));
            });
        }

        // 排序
        currentRelationList.forEach( relation -> {
            String subId = relation.getSubId();
            Integer sort = subIdList.indexOf(subId);
            relation.setSort(sort);
        });

        subRelationRepository.save(currentRelationList);
    }
}
