package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.GroupDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.product.entity.TagGroup;
import cn.rankin.productservice.repository.TagGroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class TagGroupService {

    @Autowired
    private TagGroupRepository tagGroupRepository;

    public APIResult<TagGroup> findById(String id) {
        TagGroup tagGroup = tagGroupRepository.find(id);
        if (tagGroup == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(tagGroup);
    }

    public APIResult<TagGroup> findNormalById(String id) {
        TagGroup tagGroup = tagGroupRepository.findNormalById(id, BaseStatusEnum.NORMAL);
        if (tagGroup == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(tagGroup);
    }

    public APIResult<Page<TagGroup>> search(TagGroup tagGroup, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = tagGroupRepository.count(tagGroup);

        Page<TagGroup> page = new Page<>(count, pageNo, pageSize);
        if (count == 0) {
            return APIResult.ok(page);
        }

        List<TagGroup> tagGroupList = tagGroupRepository.find(tagGroup, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(tagGroupList);

        return APIResult.ok(page);
    }

    public boolean exists(String code) {
        Long count = tagGroupRepository.countByCode(code);
        return count > 0;
    }

    @Transactional
    public APIResult<TagGroup> create(GroupDTO groupDTO) {
        String code = groupDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        TagGroup tagGroup = convert(groupDTO);
        TagGroup result = tagGroupRepository.save(tagGroup);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<TagGroup> update(GroupDTO groupDTO) {
        TagGroup tagGroup = convert(groupDTO);
        TagGroup result = tagGroupRepository.update(tagGroup);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = tagGroupRepository.deleteById(id, BaseStatusEnum.DEL);
        return APIResult.ok(count > 0);
    }
}
